#!/usr/bin/env python

# usage: java -jar bin/profilechecker.jar ProvenanceOntology.owl OWL2RLProfile 2>&1 | python bin/rlplus-justify.py

import sys
import re

explain = {}


explain['Use of non-superclass expression in position that requires a superclass expression: ObjectUnionOf(prov:Activity prov:Agent prov:Entity prov:InstantaneousEvent>) [ObjectPropertyDomain(prov:atLocation ObjectUnionOf(prov:Activity prov:Agent prov:Entity prov:InstantaneousEvent>)) ]'] = {'id' : '1', 'abbreviation' : '<a href="#atLocation" class="qname">prov:atLocation</a> rdfs:domain <span class="name">[ (prov:Activity prov:Agent prov:Entity prov:InstantaneousEvent) ]</span>', 'errortype' : '' }

explain['Use of non-superclass expression in position that requires a superclass expression: ObjectUnionOf(prov:Activity prov:Agent prov:Entity>) [ObjectPropertyRange(Annotation(prov:definition "influencer: an identifier (o1) for an ancestor entity, activity, or agent that the former depends on;") Annotation(prov:dm "http://www.w3.org/TR/2012/WD-prov-dm-20120703/prov-dm.html#term-influence") prov:wasInfluencedBy ObjectUnionOf(prov:Activity prov:Agent prov:Entity>)) ]'] = {'id' : '2', 'abbreviation' : '<a href="#wasInfluencedBy" class="qname">prov:wasInfluencedBy</a> rdfs:range <span class="name">[ (prov:Activity prov:Agent prov:Entity) ]</span>', 'errortype' : '' }

explain['Use of non-superclass expression in position that requires a superclass expression: ObjectUnionOf(prov:Delegation prov:Derivation prov:Start>) [ObjectPropertyDomain(prov:hadActivity ObjectUnionOf(prov:Delegation prov:Derivation prov:Start>)) ]'] = {'id' : '3', 'abbreviation' : '<a href="#hadActivity" class="qname">prov:hadActivity</a> rdfs:domain <span class="name">[ (prov:Delegation prov:Derivation prov:Start) ]</span>', 'errortype' : '' }

explain['Use of non-superclass expression in position that requires a superclass expression: ObjectUnionOf(prov:Association prov:End prov:Generation prov:Invalidation prov:Start prov:Usage>) [ObjectPropertyDomain(prov:hadRole ObjectUnionOf(prov:Association prov:End prov:Generation prov:Invalidation prov:Start prov:Usage>)) ]'] = {'id' : '4', 'abbreviation' : '<a href="#hadRole" class="qname">prov:hadRole</a> rdfs:domain <span class="name">[ (prov:Association prov:End prov:Generation prov:Invalidation prov:Start prov:Usage) ]</span>', 'errortype' : '' }

explain['Use of non-superclass expression in position that requires a superclass expression: ObjectUnionOf(prov:Activity prov:Agent prov:Entity>) [ObjectPropertyDomain(Annotation(prov:definition "influencee: an identifier (o2) for an entity, activity, or agent; ") Annotation(prov:dm "http://www.w3.org/TR/2012/WD-prov-dm-20120703/prov-dm.html#term-influence") prov:wasInfluencedBy ObjectUnionOf(prov:Activity prov:Agent prov:Entity>)) ]'] = {'id' : '5', 'abbreviation' : '<a href="#wasInfluencedBy" class="qname">prov:wasInfluencedBy</a> rdfs:domain <span class="name">[ (prov:Activity prov:Agent prov:Entity) ]</span>', 'errortype' : '' }

print '<div style="text-align: center">'
print '  <table id="owl-rl-violations">'
print '    <caption><a href="#owl-rl-violations">Table 3</a>: All OWL Axioms in PROV-O that do not conform to the OWL-RL profile.</caption>'
print '    <tr>'
print '      <th>Non OWL-RL PROV-O Axiom</td>'
print '    </tr>'
explained = {}
for line in sys.stdin:
   line = line.strip()
   short = re.sub('in <http://www.w3.org/ns/prov#><http://www.w3.org/TR/2012/WD-prov-o-2012....>]',']',line)
   short = re.sub('<http://www.w3.org/ns/prov#','prov:',short)
   short = re.sub('> ',' ',short)

   if short in explain:
      if short not in explained:
         html = '    <tr>\n'
         html = html + '      <td>' + explain[short]['abbreviation'] + '</td>\n'
         html = html + '    </tr> <!-- '+explain[short]['id']+' -->\n'
         explained[explain[short]['id']] = html
   else:
      print ''
      print 'TODO ------'
      print ''
      print line
      print ''
      print short
      print ''
      print ' ------ ------'
for dodad in explained:
   print explained[dodad]
print '  </table>'
print '</div>'
